'''
    File name: null.py
    Author: Bastian Kramer
    Date created: 21/10/2020
    Date last modified: 21/10/2020
    Python Version: 2.7

    This is a simple script for creating null groups as parents for selected transforms placed with same transformation
    as their future child to zero out its child's transformation.
'''

import pymel.core as pm


def createNull(obj):
    if not isinstance(obj, pm.nt.Transform):
        pm.displayError("Could not create null group for object. Object is not a transform.")
        return

    name = obj.nodeName()

    grp = pm.group(empty = True)
    grp.rename(name.replace("_ctrl", "") + "_null")

    pm.delete(pm.pointConstraint(obj, grp, mo = False))
    pm.delete(pm.orientConstraint(obj, grp, mo = False))
    pm.delete(pm.scaleConstraint(obj, grp, mo = False))

    parent = obj.getParent()
    pm.parent(obj, grp)
    if parent:
        pm.parent(grp, parent)

    return grp


objs = pm.ls(sl=1, type="transform")
if not objs:
    pm.displayError("Could not create null groups. Must at least select one transform.")
else:
    nulls = list()
    for obj in objs:
        null = createNull(obj)
        nulls.append(null)
    pm.select(nulls)
