'''
    File name: snapping.py
    Author: Bastian Kramer
    Date created: 21/10/2020
    Date last modified: 21/10/2020
    Python Version: 2.7

    This is a simple script for snapping multiple transforms to the last selected target transform.
'''


import pymel.core as pm


def snapTransform(obj, target):
    if not (isinstance(obj, pm.nt.Transform) or isinstance(target, pm.nt.Transform)):
        pm.displayError("Could not snap {obj} to {target}. One or both are no transforms.".format(obj=obj, target=target))
        return

    pCon = pm.pointConstraint(target, obj, maintainOffset = False)
    oCon = pm.orientConstraint(target, obj, maintainOffset = False)
    sCon = pm.scaleConstraint(target, obj, maintainOffset = False)

    pm.delete(pCon, oCon, sCon)


objs = pm.ls(sl=1, type="transform")[:-1]
target = pm.ls(sl=1, type="transform")[-1]
if not objs:
    pm.displayError("Could not snap transforms. Must at least select two transforms.")
else:
    for obj in objs:
        snapTransform(obj, target)
